 
def _readlines(filename, begin, end):
    lines = []
    with open(filename) as f:
        pos = 0
        for idx, line in enumerate(f):
            pos += 1
            if pos < begin:
                continue
            if pos > end:
                break
            lines.append(line.rstrip())
    return lines

def _markerpos(linestr, column, tabsize):
    assert len(linestr) > column
    return len(linestr[:column].expandtabs(tabsize))

def _markerline(marker_pos):
    return ''.rjust(marker_pos) + '^'

def _prepend_text(lines, text):
    return [text + line for line in lines]

def _prepend_linenums(lines, begin, end):
    result = []
    margin = len(str(end))
    for idx,line in enumerate(lines):
        num = idx + begin
        result.append("{0: <{width}} {1}".format(str(num), line, width=margin))
    return result


padding = '| '

def format_sloc(token, off):
    assert token.source_file is not None
    begin = max(1, token.line - off)
    end = token.line + off
    lines = _readlines(token.source_file, begin, end)
    end = begin + len(lines) - 1

    # First inject the marker line into the source lines and then
    # do all the processing with all lines. Doing so one does not need
    # to track the number of inserted whitespaces because the marker
    # will be processed together with the other lines.
    idx = token.line - begin
    pos = _markerpos(lines[idx], token.column, 4)
    margin = len(str(end)) + 1
    marker = _markerline(pos + margin)

    lines = _prepend_linenums(lines, begin, end)
    lines.insert(idx+1, marker)
    lines = _prepend_text(lines, padding)
    return lines
