import antlr3
from output.IVLLexer import IVLLexer as Lexer
from output.IVLParser import IVLParser as Parser, ParserException

def _parse_source(char_stream, filepath, inputtext):
    lexer = Lexer(char_stream)
    tokens = antlr3.CommonTokenStream(lexer)
    parser = Parser(tokens)
    try:
        prog = parser.prog(filepath, inputtext)
    except ParserException:
        raise

    if lexer.getNumberOfSyntaxErrors() > 0:
        raise ParserException("lexer failed", inputfile=filepath, inputtext=inputtext)
    if parser.getNumberOfSyntaxErrors() > 0:
        raise ParserException("parser failed", inputfile=filepath, inputtext=inputtext)

    return prog


def parse_file(filepath):
    import os
    filepath = os.path.abspath(filepath)
    char_stream = antlr3.ANTLRFileStream(filepath)
    return _parse_source(char_stream, filepath, None)

def parse_string(inputtext):
    char_stream = antlr3.ANTLRStringStream(inputtext)
    return _parse_source(char_stream, None, inputtext)

def parse_stdin():
    import sys
    char_stream = antlr3.ANTLRInputStream(sys.stdin)
    return _parse_source(char_stream, None, None)
